/*! \file main.cpp
*/

#include "Galil.h"
#include <iostream>
#include <iomanip>
#include <Windows.h> //Sleep
using namespace std;

int run(int argc, char *argv[])
{
	try
	{
		cout << Galil::libraryVersion() << '\n';

		for (string s : Galil::addresses())
			cout << s << '\n';

		//Galil g("GALILPCI1");
		Galil g("192.168.0.100");

		try
		{
			g.command("xx"); //will throw string
		}
		catch (string s)
		{
			cout << s << endl;
		}

		cout << g.connection() << '\n';
		cout << "MGTIME: [" << g.command("MG TIME") << "]\n"; //brackets to visually check trimming
		//g.command("UI 0");
		//cout << "Interrupt: " << g.interrupt() << '\n'; //30000 Bug 2047 - UDP interrupt payload byte count incorrect
		g.programDownload("WT 600\rMG \"Message from DMC code\", TIME\rEN");
		cout << '\n' << g.programUpload() << '\n' << '\n';
		g.command("XQ");

		Sleep(1000);
		cout << g.message(700);


		vector<double> down;
		down.push_back(3.1415);
		down.push_back(2.7183);
		down.push_back(1.6180);
		down.push_back(42);
		g.arrayDownload(down, "mynums");
		g.arrayUploadFile(); //uses the default arrays.csv
		g.command("DA *[]");
		g.arrayDownloadFile(); //uses the default arrays.csv
		vector<double> up = g.arrayUpload("mynums");
		for (auto d : up) cout << d << " ";
		cout << '\n';

		// g.firmwareDownloadFile("\\\\rd3\\applications2\\firmware\\hex2\\4000.dmc\\dmc-4000-r12b-bck.hex");
		// cout << g.connection() << '\n';
		// g.firmwareDownloadFile("\\\\rd3\\applications2\\firmware\\hex2\\4000.dmc\\dmc-4000-r12b.hex");
		// cout << g.connection() << '\n';

		g.write("MG TIME \r");
		cout << g.read();
		cout << "\n\n";

		//Data record


		vector<string> sources = g.sources();
		vector<char> r = g.record("QR");
		for (auto s : sources)
		{
			cout << left << setw(10)
				<< g.sourceValue(r, s)
				<< setw(10) << s

				<< g.source("Description", s)
				<< ", " << g.source("Units", s)
				<< ", S" << g.source("Scale", s)
				<< " O" << g.source("Offset", s)
				<< '\n';
		}
		cout << "\n\n";

		//modify source properties
		/*g.setSource("Units", "TIME", "seconds");
		g.setSource("Description", "TIME", "Seconds counter");
		g.setSource("Scale", "TIME", "1024");

		cout << "TIME" << ""
		<< ", " << g.source("Description", "TIME")
		<< ":" << g.sourceValue(g.record(), "TIME")
		<< " " << g.source("Units", "TIME")
		<< " S:" << g.source("Scale", "TIME")
		<< " O:" << g.source("Offset", "TIME")
		<< '\n';*/


		return 0;
	}
	catch (string s)
	{
		cout << s;
		return 1;
	}

}


int main(int argc, char *argv[])
{
	int rc = run(argc, argv);
#ifdef _DEBUG
	cout << "\n\nPress enter to exit...\n";
	getchar();
#endif
	return rc;
}